package mjc.gc;

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.PrintWriter;

import mjc.compiler.MJCException;
import mjc.lib.TDC;
import mjc.lib.TDS;
import mjc.lib.VARIABLE;
import mjc.lib.classe.CLASSE;
import mjc.lib.classe.METHODE;
import mjc.lib.classe.PARAM;
import mjc.lib.type.DTYPE;

public abstract class AbstractMachine {
	protected abstract String getSuffixe();

	public void writeCode(String fname, String code) throws MJCException {
		try {
			String asmName = fname + "." + getSuffixe();
			System.err.println("Ecriture du code dans " + asmName);
			PrintWriter pw = new PrintWriter(new FileOutputStream(asmName));
			pw.print(code);
			pw.close();
		} catch (FileNotFoundException e) {
			throw new MJCException(e.getMessage());
		}
	}
	
	/** Génère la TDV pour l'ensemble du programme. */
	public abstract String genererTDV(TDC classes);
	
	/** Génère le code pour l'appel à <b>main</b>. */
	public abstract String genererAppelMain(TDC classe);

	/** Libère la mémoire de <i>taille</i>.*/
	public abstract String genFree(int taille);
	
	/** Génère le corps d'une méthode.
	 * 
	 * @param classe Nom de la classe
	 * @param methode Information concernant la méthode
	 * @param code Code du bloc
	 */
	public abstract String genMethode(String classe, METHODE methode, String code);
	
	/** Retour d'une fonction.
	 * 
	 * @param m La fonction dont on génère le retour.
	 * @param locales Table des locales de la fonction.
	 */
	public abstract String genRetour(METHODE m, TDS locales);
	
	/** Génère le corps d'un constructeur (appel au constructeur par défaut).
	 * 
	 * @param classe Nom de la classe
	 * @param constructeur Information concernant le constructeur appelé
	 * @param code Code du constructeur
	 */
	public abstract String genConstructeur(String classe, METHODE constructeur, String code);
	
	/** Génère le code pour un constructeur par défaut. */
	public abstract String genConstructeurDefaut(CLASSE c);
	
	/** Génère le code appelé en cas de NPE. */
	public abstract String genererCodeNPE();
	
	/** Générer le code de l'appel d'une méthode.
	 * 
	 * @param classe Classe
	 * @param m Méthode
	 */
	public abstract String genAppelMeth(CLASSE classe, METHODE m);
	
	/** Code une écriture en mémoire.
	 * 
	 * @param adresse Adresse de l'écriture
	 * @param expression Valeur à stocker
	 * @param type Type (utilisé pour la taille)
	 */
	public abstract String genEcritureMem(String adresse, String expression, DTYPE type);
	
	/** Code une lecture mémoire.
	 * 
	 * @param code Code pour obtenir l'adresse
	 * @param type Type à lire (~taille de la donnée)
	 */
	public abstract String genLectureMem(String code, DTYPE type);
	
	/** Code une lecture en mémoire. Teste si le code "code" la valeur ou l'adresse. */
	public abstract String genLectureMem(String code, DTYPE type, boolean estValeur);

	/** Génère une affectation (avec déclaration). */
	public abstract String genAffect(VARIABLE variable);
	
	/** Déclaration d'une variable (sans affectation). */
	public abstract String genDecl(VARIABLE var);

	/** Allocation de mémoire.
	 * 
	 * @param type Type à allouer
	 * @param m Méthode (constructeur) à appeller (null si celui par défaut)
	 * @param code Le code pour les arguments du constructeur
	 */
	public abstract String genAppelConstructeur(DTYPE type, METHODE m, String code);
	
	/** Vérifie si un pointeur n'est pas <b>null</b>. */
	public abstract String genVerifNPE();
	
	/** Génère l'adresse d'une locale. */
	public abstract String genAdresseVariable(VARIABLE var);

	/** Génère l'adresse d'un paramètre de méthode. */
	public abstract String genAdresseParam(PARAM param);

	/** Génère l'adresse d'un attribut de classe. */
	public abstract String genAdresseAttribut(PARAM attribut);
	
	/** Génère la valeur d'une locale. */
	public abstract String genAccesVariable(VARIABLE var);

	/** Génère la valeur d'un paramètre de méthode. */
	public abstract String genAccesParam(PARAM param);

	/** Génère la valeur d'un attribut de classe. */
	public abstract String genAccesAttribut(PARAM attribut);
	
	/** Empile <i>this</i> en tête de pile.
	 *
	 *@param m Methode dont on cherche à récupérer <i>this</i>.
	 */
	public abstract String genEmpilerThis(METHODE m);

	/** Génère l'affichage de l'expression produite par le code. */
	public abstract String genAffichage(String code, DTYPE type);
	
	/** Génère une boucle <i>tant que</i>.
	 * 
	 * @param condition Code de la condition
	 * @param boucle Code de la boucle
	 */
	public abstract String genTq(String condition, String boucle) ;

	/** Génère un <i>si, alors, sinon</i>.
	 * 
	 * @param condition Code de la condition.
	 * @param alors Code du alors.
	 * @param sinon Code du sinon
	 */
	public abstract String genSi(String condition, String alors, String sinon);
	
	/** Le pointeur <b>null</b>. */
	public abstract String genNull();
	
	/** La valeur booléenne <b>faux</b>. */
	public abstract String genFaux();
	
	/** La valeur booléenne <b>vrai/b>. */
	public abstract String genVrai();

	/** Charge un entier.
	 * 
	 * @param valeur Valeur de l'entier.
	 */
	public abstract String genEntier(String valeur);
	
	/** Code de l'addition. */
	public abstract String genAddition();

	/** Code de la soustraction. */
	public abstract String genMoins();

	/** Code du <i>modulo</i>. */
	public abstract String genMod();

	/** Code de la multiplication. */
	public abstract String genMult();

	/** Code de la division. */
	public abstract String genDiv();

	/** Code de l'opérateur <i>></i>. */
	public abstract String genSup();
	
	/** Code de l'opérateur <i>>=</i>.*/
	public abstract String genSupEg();

	/** Code de l'opérateur <i>==</i>. */
	public abstract String genEg();
	
	/** Code de l'opérateur <i>!=<i>. */
	public abstract String genNeq();

	/** Code de l'opérateur <i><</i>. */
	public abstract String genInf();

	/** Code de l'opérateur <i><=</i>. */
	public abstract String genInfEg();
	
	/** Code du <i>OU</i>. */
	public abstract String genOu();
	
	/** Code du <i>ET</i>. */
	public abstract String genEt();

	/** Code du <i>NON</i>. */
	public abstract String genNon();

	/** Code l'opérateur unaire <i>-</i>. */
	public abstract String genOpp();
}
